﻿using System;
using System.Linq;
using System.Windows.Forms;
using System.Collections.ObjectModel;

/* Test application for Logistic Regression on Iris data set */
/*                  www.mosismath.com                        */

namespace WindowsFormsApplication1
{
    
    public partial class Mainwin : Form
    {

        int samples = 60;
        const int features_in = 4;
        const int features_out = 2 * features_in;    
      
        public Collection<double[]> rise = new Collection<double[]>();
        double[] offset = new double[3];
        Data data = new Data();
       
      
        double calc_probability(double[] x, double[] rise, double offset)
        {
            return 1 / (1 + 1 / Math.Exp(F_x(x, rise, offset)));
        }

        double F_x(double[] x, double[] rise, double offs)
        {
            int i;
            double res = offs;
            for (i = 0; i < features_out / 2; i++)
            {
                res = res + x[i] * x[i] * rise[2 * i] + x[i] * rise[2 * i + 1];
            }
            return res;
        }

        public Mainwin()
        {
            int i, j;
            DataGridViewCell cell;

            InitializeComponent();
            data.readData(Application.StartupPath + "\\Data_iris_test.csv");
            samples = data.values.Count;

            dgData.RowCount = samples;
            dgData.ColumnCount = features_in + 3;
            for (j = 0; j < features_in; j++)
                dgData.Columns[j].HeaderText = "x" + (j+1).ToString();
            dgData.Columns[j].HeaderText = "Id";
            dgData.Columns[j+1].HeaderText = "Flower";
            dgData.Columns[j + 2].HeaderText = "prob. [%]";
            for (i = 0; i < samples; i++)
            {
                dgData.Rows[i].HeaderCell.Value = (i+1).ToString();
    
                for (j = 0; j <= 4; j++)
                {
                    cell = dgData[j, i];
                    cell.Value = Math.Round(data.values.ElementAt(i)[j], 4).ToString();
                }
            }
            double[] line = new double[features_out];
            line[0] = 0.5680312627537083; // Iris-versicolor
            line[1] = -5.3954678636904729;
            line[2] = 0.44293119469303405;
            line[3] = -2.0507009129128559;
            line[4] = -1.3667510103331613;
            line[5] = 10.334452116078955;
            line[6] = -3.2710795170877502;
            line[7] = 5.1119839409034764;
            rise.Add(line);
            offset[0] = -2.953125;  // cost = 0.0112

            line = new double[features_out];
            line[0] = 0.1576914606097137; // Iris_setosa
            line[1] = 0.11451444296925899;
            line[2] = 0.46501487429496885;
            line[3] = 0.13679861493200013;
            line[4] = -0.83229051009806876;
            line[5] = -0.11448436500795058;
            line[6] = -0.093933076641649821;
            line[7] = -0.053559051471672593;
            rise.Add(line);
            offset[1] = -2.5;  // cost = 2.21E-05

            line = new double[features_out];
            line[0] = -0.37177631181499771; // Iris_virginica
            line[1] = -2.0763316437336172;
            line[2] = -1.1654852486586853;
            line[3] = -0.90905595883895385;
            line[4] = 1.5813873303492454;
            line[5] = -1.4923688608135275;
            line[6] = 3.7812812345532012;
            line[7] = 0.55317350173826108;
            rise.Add(line);
            offset[2] = -2.328125;  // cost = 0.0067
        }



        private void bt_Calc_Click(object sender, EventArgs e)
        {
            int i;
            double[] x;
            double probability = 0;
            string flowerName = "";
            DataGridViewCell cell;


            for (i = 0; i < samples; i++)
            {
                x = data.values.ElementAt(i);
                probability = calc_probability(x, rise.ElementAt(0), offset[0]);
                if (probability > 0.5)
                {
                    flowerName = "Iris-versicolor";
                }
                else
                {
                    probability = calc_probability(x, rise.ElementAt(1), offset[1]);
                    if (probability > 0.5)
                    {
                        flowerName = "Iris_setosa";
                    }
                    else
                    {
                        probability = calc_probability(x, rise.ElementAt(2), offset[2]);
                        if (probability > 0.5)
                        {
                            flowerName = "Iris_virginica";
                        }
                        else
                        {
                            flowerName = "Not recognized";
                        }
                    }
                }
                // output the result
                cell = dgData[4 + 1, i];
                cell.Value = flowerName;
                cell = dgData[4 + 2, i];
                cell.Value = Math.Round(probability * 100, 2); 
            }
        }
    }
}
